
#include "CYRF6936.h"
#include "Arduino.h"

CYRF6936::CYRF6936(unsigned char pinCS, unsigned char pinSCK, unsigned char pinMOSI,
    unsigned char pinMISO)
    : m_pinCS(pinCS)
    , m_pinSCK(pinSCK)
    , m_pinMOSI(pinMOSI)
    , m_pinMISO(pinMISO)
{
}

CYRF6936::~CYRF6936()
{
}

void CYRF6936::init()
{
    // setup pin mode
    pinMode(m_pinMOSI, OUTPUT);
    pinMode(m_pinMISO, INPUT);
    pinMode(m_pinSCK, OUTPUT);
    pinMode(m_pinCS, OUTPUT);

    // disable device
    digitalWrite(m_pinCS, HIGH);

    // setup SPI control register: SPCR = 01010000
    // interrupt disabled, spi enabled, msb 1st, master, clk low when idle,
    // sample on rising edge of clk, system clock rate fastest
    SPCR = (1<<SPE) | (1<<MSTR);

    // clear data registers
    byte clr = SPSR;
    clr = SPDR;
}

unsigned char CYRF6936::sendByte(unsigned char data)
{
    // enable device
    digitalWrite(m_pinCS, LOW);

    // wait for device
    //while (digitalRead(m_pinMISO) == HIGH) {
    //};

    // send byte
    unsigned char result = spiTransfer(data);

    // disable device
    digitalWrite(m_pinCS, HIGH);

    // return result
    return result;
}

unsigned char CYRF6936::sendCommand(unsigned char command, unsigned char data)
{
    // enable device
    digitalWrite(m_pinCS, LOW);

    // wait for device
    //while (digitalRead(m_pinMISO) == HIGH) {
    //};

    // send command byte
    spiTransfer(command);

    // send data byte
    unsigned char result = spiTransfer(data);

    // disable device
    digitalWrite(m_pinCS, HIGH);

    // return result
    return result;
}

unsigned char CYRF6936::sendCommand2(unsigned char command, unsigned char data, unsigned char data2)
{
    // enable device
    digitalWrite(m_pinCS, LOW);

    // wait for device
    //while (digitalRead(m_pinMISO) == HIGH) {
    //};

    // send command byte
    spiTransfer(command);

	// send first data byte
	spiTransfer(data);
	
    // send data byte
    unsigned char result = spiTransfer(data2);

    // disable device
    digitalWrite(m_pinCS, HIGH);

    // return result
    return result;
}

unsigned char CYRF6936::sendStrobeCommand(unsigned char command)
{
    // send command
    return sendByte(command);
}

unsigned char CYRF6936::sendBurstCommand(unsigned char command, unsigned char* data,
    unsigned char length)
{
    // enable device
    digitalWrite(m_pinCS, LOW);

    // wait for device
    //while (digitalRead(m_pinMISO) == HIGH) {
    //};

    // send command byte
    spiTransfer(command);

    unsigned char result = 0;

    // send data bytes
    for (int i=0; i<length; ++i) {
        result = spiTransfer(data[i]);
        //data[i] = result;
    }

    // disable device
    digitalWrite(m_pinCS, HIGH);

    // return result
    return result;
}

unsigned char CYRF6936::receiveBurstData(unsigned char command, unsigned char* data, unsigned char length)
{
    // enable device
    digitalWrite(m_pinCS, LOW);

    // wait for device
    //while (digitalRead(m_pinMISO) == HIGH) {
    //};

    // send command byte
    spiTransfer(command);

    unsigned char result = 0;

    // send data and receive bytes
    for (int i=0; i<length; ++i) {
        result = spiTransfer(data[i]);
        data[i] = result;
    }

    // disable device
    digitalWrite(m_pinCS, HIGH);

    // return result
    return result;
}

unsigned char CYRF6936::spiTransfer(volatile unsigned char data)
{
    // start transmission
    SPDR = data;

    // wait for end of transmission
    while (!(SPSR & (1<<SPIF))) {
    };

    // return received byte
    return SPDR;
}
